@@ -6,9 +6,6 @@
 
 
 Archive::Archive(RAROptions *InitCmd)
-#ifndef SHELL_EXT
-:SubDataIO(NULL)
-#endif
 {
   Cmd=InitCmd==NULL ? &DummyCmd:InitCmd;
   OpenShared=Cmd->OpenShared;
@@ -50,6 +47,7 @@ Archive::Archive(RAROptions *InitCmd)
   NewArchive=false;
 
   SilentOpen=false;
+
 }
 
 
@@ -133,12 +131,18 @@ bool Archive::IsArchive(bool EnableBroken)
   }
   else
   {
-    Array<char> Buffer(0x40000);
+    Array<char> Buffer(MAXSFXSIZE);
     long CurPos=int64to32(Tell());
     int ReadSize=Read(&Buffer[0],Buffer.Size()-16);
     for (int I=0;I<ReadSize;I++)
       if (Buffer[I]==0x52 && IsSignature((byte *)&Buffer[I]))
       {
+        if (OldFormat && I>0 && CurPos<28 && ReadSize>31)
+        {
+          char *D=&Buffer[28-CurPos];
+          if (D[0]!=0x52 || D[1]!=0x53 || D[2]!=0x46 || D[3]!=0x58)
+            continue;
+        }
         SFXSize=CurPos+I;
         Seek(SFXSize,SEEK_SET);
         if (!OldFormat)
@@ -177,6 +181,19 @@ bool Archive::IsArchive(bool EnableBroken)
   Protected=(NewMhd.Flags & MHD_PROTECT)!=0;
   Encrypted=(NewMhd.Flags & MHD_PASSWORD)!=0;
 
+  if (NewMhd.EncryptVer>UNP_VER)
+  {
+#ifdef RARDLL
+    Cmd->DllError=ERAR_UNKNOWN_FORMAT;
+#else
+    ErrHandler.SetErrorCode(WARNING);
+  #if !defined(SILENT) && !defined(SFX_MODULE)
+      Log(FileName,St(MUnknownMeth),FileName);
+      Log(FileName,St(MVerRequired),NewMhd.EncryptVer/10,NewMhd.EncryptVer%10);
+  #endif
+#endif
+    return(false);
+  }
 #ifdef RARDLL
   SilentOpen=true;
 #endif
@@ -236,3 +253,5 @@ int Archive::GetRecoverySize(bool Required)
 #endif
 
 
+
+
