@@ -14,15 +14,17 @@ int main(int argc, char *argv[])
 #ifdef _UNIX
   setlocale(LC_ALL,"");
 #endif
+
 #if defined(_EMX) && !defined(_DJGPP)
   uni_init(0);
 #endif
-#ifndef SFX_MODULE
-  setbuf(stdout,NULL);
 
-  #ifdef _EMX
-    EnumConfigPaths(argv[0],-1);
-  #endif
+#if !defined(_SFX_RTL_) && !defined(_WIN_32)
+  setbuf(stdout,NULL);
+#endif
+
+#if !defined(SFX_MODULE) && defined(_EMX)
+  EnumConfigPaths(argv[0],-1);
 #endif
 
   ErrHandler.SetSignalHandlers(true);
